@@ -54,6 +54,11 @@ def test_parse_inline_html():
     )
     assert '<span><strong>' in ret
 
+    ret = mistune.markdown(
+        '<a>http://lepture.com</a>', parse_inline_html=True, escape=False
+    )
+    assert 'href' not in ret
+
 
 def test_parse_block_html():
     ret = mistune.markdown(
@@ -79,3 +84,8 @@ def test_trigger_more_cases():
 def test_not_escape_block_tags():
     text = '<h1>heading</h1> text'
     assert text in mistune.markdown(text, escape=False)
+
+
+def test_not_escape_inline_tags():
+    text = '<a name="top"></a>'
+    assert text in mistune.markdown(text, escape=False)
